>From 966a00d4a929b8ea14f75bde01f60e34061a6da3 Mon Sep 17 00:00:00 2001
From: Roiy <roiy@cs.umanitoba.ca>
Date: Fri, 4 Oct 2013 13:54:45 -0500
Subject: [PATCH 5/9] New button infrastructure

---
 modules/gui/qt4/actions_manager.cpp       |  44 ++++++++++
 modules/gui/qt4/actions_manager.hpp       |  11 +++
 modules/gui/qt4/components/controller.cpp | 129 +++++++++++++++++++++++++++++-
 modules/gui/qt4/components/controller.hpp |  34 +++++++-
 modules/gui/qt4/dialogs/toolbar.cpp       |  54 +++++++++++++
 5 files changed, 266 insertions(+), 6 deletions(-)

diff --git a/modules/gui/qt4/actions_manager.cpp b/modules/gui/qt4/actions_manager.cpp
index fa3703b..d6f4413 100644
--- a/modules/gui/qt4/actions_manager.cpp
+++ b/modules/gui/qt4/actions_manager.cpp
@@ -35,6 +35,8 @@
 #include "main_interface.hpp" /* Show playlist */
 #include "components/controller.hpp" /* Toggle FSC controller width */
 
+#include <QMessageBox> //ROIY REMOVE
+
 ActionsManager * ActionsManager::instance = NULL;
 
 ActionsManager::ActionsManager( intf_thread_t * _p_i, QObject *_parent )
@@ -85,6 +87,15 @@ void ActionsManager::doAction( int id_action )
         case SKIP_FW_ACTION:
             skipForward();
             break;
+        case PREV_SEGMENT_ACTION:
+            prevSegment();
+            break;
+        case NEXT_SEGMENT_ACTION:
+            nextSegment();
+            break;
+        case AUTO_JUMP_ACTION:
+            THEMIM->toggleAutoJump();
+            break;
         case QUIT_ACTION:
             THEDP->quit();  break;
         case RANDOM_ACTION:
@@ -97,6 +108,25 @@ void ActionsManager::doAction( int id_action )
             if( p_intf->p_sys->p_mi )
                 p_intf->p_sys->p_mi->getFullscreenControllerWidget()->toggleFullwidth();
             break;
+        case ANNOTATIONS_ACTION:
+            THEDP->annotationsDialog(); break;
+        case EVENT_LAYERS_ACTION:
+            THEDP->eventLayersDialog(); break;
+        case NEW_EVENT_ACTION:
+            THEDP->eventPropertiesDialog(); break;
+        case EDIT_EVENT_ACTION:
+            THEDP->editEventDialog(); break;
+        case DELETE_EVENT_ACTION:
+            THEDP->deleteEventDialog(); break;
+        case SAVE_EVENTS_ACTION:
+            {QMessageBox msgBox;
+            if ( THEMIM->getIM()->saveEnhancementData() == VLC_SUCCESS )
+                msgBox.setText( "<p align='center'>File saved.</p>" );
+            else
+                msgBox.setText( "<p align='center'>Error! File not saved!</p>" );
+            msgBox.exec();
+
+            break;}  //ROIY CLEAN THIS
         default:
             msg_Dbg( p_intf, "Action: %i", id_action );
             break;
@@ -205,3 +235,17 @@ void ActionsManager::skipBackward()
         THEMIM->getIM()->jumpBwd();
 }
 
+void ActionsManager::prevSegment()
+{
+    input_thread_t *p_input = THEMIM->getInput();
+    if( p_input )
+        THEMIM->getIM()->prevSegment();
+}
+
+void ActionsManager::nextSegment()
+{
+    input_thread_t *p_input = THEMIM->getInput();
+    if( p_input )
+        THEMIM->getIM()->nextSegment();
+}
+
diff --git a/modules/gui/qt4/actions_manager.hpp b/modules/gui/qt4/actions_manager.hpp
index a694579..474f2bf 100644
--- a/modules/gui/qt4/actions_manager.hpp
+++ b/modules/gui/qt4/actions_manager.hpp
@@ -56,6 +56,15 @@ typedef enum actionType_e
     LOOP_ACTION,
     INFO_ACTION,
     OPEN_SUB_ACTION,
+    ANNOTATIONS_ACTION,
+    EVENT_LAYERS_ACTION,
+    PREV_SEGMENT_ACTION,
+    NEXT_SEGMENT_ACTION,
+    AUTO_JUMP_ACTION,
+    NEW_EVENT_ACTION,
+    EDIT_EVENT_ACTION,
+    DELETE_EVENT_ACTION,
+    SAVE_EVENTS_ACTION
 } actionType_e;
 
 class ActionsManager : public QObject
@@ -90,6 +99,8 @@ public slots:
     void record();
     void skipForward();
     void skipBackward();
+    void prevSegment();
+    void nextSegment();
 protected slots:
     void fullscreen();
     void snapshot();
diff --git a/modules/gui/qt4/components/controller.cpp b/modules/gui/qt4/components/controller.cpp
index a5b3856..459c541 100644
--- a/modules/gui/qt4/components/controller.cpp
+++ b/modules/gui/qt4/components/controller.cpp
@@ -43,6 +43,7 @@
 #include "util/customwidgets.hpp"                       /* qEventToKey */
 
 #include "adapters/seekpoints.hpp"
+#include "adapters/seeksegments.hpp"
 
 #include <QToolButton>
 #include <QHBoxLayout>
@@ -312,8 +313,20 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
         NORMAL_BUTTON( EXTENDED );
         }
         break;
-    case PLAYLIST_BUTTON:{
-        NORMAL_BUTTON( PLAYLIST );
+    case ANNOTATIONS_BUTTON:{
+        NORMAL_BUTTON( ANNOTATIONS );
+        }
+        break;
+    case EVENT_LAYERS_BUTTON:{
+        NORMAL_BUTTON( EVENT_LAYERS );
+        }
+        break;
+    case PREV_SEGMENT_BUTTON:{
+        NORMAL_BUTTON( PREV_SEGMENT );
+        }
+        break;
+    case NEXT_SEGMENT_BUTTON:{
+        NORMAL_BUTTON( NEXT_SEGMENT );
         }
         break;
     case SNAPSHOT_BUTTON:{
@@ -365,6 +378,37 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
         widget = slider;
         }
         break;
+    case META_SLIDER: {
+        MetaSlider *slider = new MetaSlider( Qt::Horizontal, NULL, p_intf, !b_shiny );
+        
+        SeekPoints *chapters = new SeekPoints( this, p_intf );
+        CONNECT( THEMIM->getIM(), chapterChanged( bool ), chapters, update() );
+        slider->setChapters( chapters );
+
+        SeekSegments *events = new SeekSegments( this, p_intf );
+        slider->setEvents( events );
+        CONNECT( THEMIM->getIM(), eventsChanged( bool ), events, updateMain() );
+        CONNECT( THEMIM->getIM(), layersChanged( ), events, updateMain() );
+        CONNECT( THEMIM->getIM(), layersChanged( ), slider, repaint() );
+        CONNECT( THEMIM->getIM(), eventEditToggled( ), slider, toggleEditing() );
+        CONNECT( slider, switchEdit( bool, int64_t ), THEMIM->getIM(), switchEditMode( bool, int64_t ) );
+        CONNECT( THEMIM->getIM(), updateEventTimes( int64_t, int64_t ), slider, setEventTimes( int64_t, int64_t ) );
+        
+
+        /* Update the position when the IM has changed */
+        CONNECT( THEMIM->getIM(), positionUpdated( float, int64_t, int ),
+                slider, setPosition( float, int64_t, int ) );
+        /* And update the IM, when the position has changed */
+        CONNECT( slider, sliderDragged( float ),
+                 THEMIM->getIM(), sliderUpdate( float ) );
+        CONNECT( THEMIM->getIM(), cachingChanged( float ),
+                 slider, updateBuffering( float ) );
+        /* Give hint to disable slider's interactivity when useless */
+        CONNECT( THEMIM->getIM(), inputCanSeek( bool ),
+                 slider, setSeekable( bool ) );
+        widget = slider;
+        }
+        break;
     case MENU_BUTTONS:
         widget = discFrame();
         widget->hide();
@@ -387,6 +431,23 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
             widget = timeLabel;
         }
         break;
+    case SEGMENTS_TIME_LABEL:
+        {
+            widget = new SegmentsTimeLabel( p_intf );
+            CONNECT( THEMIM->getIM(), layersChanged( ), widget, clearCache() );
+        }
+        break;
+    case SEGMENTS_TIME_LABEL_ELAPSED:
+        {
+            widget = new SegmentsTimeLabel( p_intf, TimeLabel::Elapsed );
+        }
+        break;
+    case SEGMENTS_TIME_LABEL_REMAINING:
+        {
+            widget = new SegmentsTimeLabel( p_intf, TimeLabel::Remaining );
+            CONNECT( THEMIM->getIM(), layersChanged( ), widget, clearCache() );
+        }
+        break;
     case SPLITTER:
         {
             QFrame *line = new QFrame;
@@ -426,6 +487,16 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
         ENABLE_ON_INPUT( SKIP_FWButton );
         }
         break;
+    case AUTO_JUMP_BUTTON: {
+        AutoJumpButton *autoJumpButton = new AutoJumpButton;
+        setupButton( autoJumpButton );
+        BUTTON_SET_BAR( autoJumpButton );
+        CONNECT_MAP_SET( autoJumpButton, AUTO_JUMP_ACTION );
+        CONNECT( THEMIM, autoJumpChanged( bool ),
+                 autoJumpButton, updateButtonIcons( bool ) );
+        widget = autoJumpButton;
+        }
+        break;
     case QUIT_BUTTON: {
         NORMAL_BUTTON( QUIT );
         }
@@ -481,6 +552,9 @@ QWidget *AbstractController::createWidget( buttonType_e button, int options )
     case TIME_LABEL_REMAINING:
         widget = new TimeLabel( p_intf, TimeLabel::Remaining );
         break;
+    case EVENTS_EDITOR:
+        widget = eventsFrame();
+        break;
     default:
         msg_Warn( p_intf, "This should not happen %i", button );
         break;
@@ -563,6 +637,41 @@ QFrame *AbstractController::discFrame()
     return discFrame;
 }
 
+QFrame *AbstractController::eventsFrame()
+{
+    /** Events and annotations handling */
+    QFrame *eventsFrame = new QFrame( this );
+
+    QHBoxLayout *eventsLayout = new QHBoxLayout( eventsFrame );
+    eventsLayout->setSpacing( 0 ); eventsLayout->setMargin( 0 );
+
+    QToolButton *newEventButton = new QToolButton( eventsFrame );
+    setupButton( newEventButton );
+    BUTTON_SET_BAR2( newEventButton, util/create_event, qtr("Create new event") );
+    CONNECT_MAP_SET( newEventButton, NEW_EVENT_ACTION );
+    eventsLayout->addWidget( newEventButton );
+
+    QToolButton *editEventButton = new QToolButton( eventsFrame );
+    setupButton( editEventButton );
+    BUTTON_SET_BAR2( editEventButton, util/edit_event, qtr("Edit existing event") );
+    CONNECT_MAP_SET( editEventButton, EDIT_EVENT_ACTION );
+    eventsLayout->addWidget( editEventButton );
+
+    QToolButton *deleteEventButton = new QToolButton( eventsFrame );
+    setupButton( deleteEventButton );
+    BUTTON_SET_BAR2( deleteEventButton, util/delete_event, qtr("Delete event") );
+    CONNECT_MAP_SET( deleteEventButton, DELETE_EVENT_ACTION );
+    eventsLayout->addWidget( deleteEventButton );
+
+    QToolButton *saveEventsData = new QToolButton( eventsFrame );
+    setupButton( saveEventsData );
+    BUTTON_SET_BAR2( saveEventsData, util/save_events, qtr("Save events data to file") );
+    CONNECT_MAP_SET( saveEventsData, SAVE_EVENTS_ACTION );
+    eventsLayout->addWidget( saveEventsData );
+
+    return eventsFrame;
+}
+
 QFrame *AbstractController::telexFrame()
 {
     /**
@@ -716,6 +825,22 @@ InputControlsWidget::InputControlsWidget( intf_thread_t *_p_i, QWidget *_parent
     QString line = getSettings()->value( "MainWindow/InputToolbar", INPT_TB_DEFAULT ).toString();
     parseAndCreate( line, controlLayout );
 }
+
+MetaLayerControlsWidget::MetaLayerControlsWidget( intf_thread_t *_p_i, QWidget *_parent ) :
+                                     AbstractController( _p_i, _parent )
+{
+    RTL_UNAFFECTED_WIDGET
+    controlLayout = new QHBoxLayout( this );
+    controlLayout->setMargin( 0 );
+    controlLayout->setSpacing( 0 );
+#ifdef DEBUG_LAYOUT
+    setStyleSheet( "background: green ");
+#endif
+
+    QString line = getSettings()->value( "MainWindow/MetaLayerToolbar", META_TB_DEFAULT ).toString();
+    parseAndCreate( line, controlLayout );
+}
+
 /**********************************************************************
  * Fullscrenn control widget
  **********************************************************************/
diff --git a/modules/gui/qt4/components/controller.hpp b/modules/gui/qt4/components/controller.hpp
index bfe68e9..ea8a7c6 100644
--- a/modules/gui/qt4/components/controller.hpp
+++ b/modules/gui/qt4/components/controller.hpp
@@ -35,9 +35,10 @@
 #include <QSizeGrip>
 
 #define MAIN_TB1_DEFAULT "64;39;64;38;65"
-#define MAIN_TB2_DEFAULT "0-2;64;3;1;4;64;7;9;64;10;20;19;64-4;37;65;35-4"
+#define MAIN_TB2_DEFAULT "0-2;64;3;1;4;64;7;9;64;10;20;19;64;27;28;26;29;64-4;37;65;35-4"
 #define ADV_TB_DEFAULT "12;11;13;14"
 #define INPT_TB_DEFAULT "43;33-4;44"
+#define META_TB_DEFAULT "47;45-4;48"
 #define FSC_TB_DEFAULT "0-2;64;3;1;4;64;37;64;38;64;8;65;25;35-4;34"
 
 #define I_PLAY_TOOLTIP N_("Play\nIf the playlist is empty, open a medium")
@@ -53,6 +54,7 @@ class QVBoxLayout;
 class QAbstractSlider;
 class QAbstractButton;
 class SeekSlider;
+class MetaSlider;
 class QToolButton;
 
 class VolumeClickHandler;
@@ -89,7 +91,13 @@ typedef enum buttonType_e
     NEXT_BUTTON,
     OPEN_SUB_BUTTON,
     FULLWIDTH_BUTTON,
+    ANNOTATIONS_BUTTON,   
+    PREV_SEGMENT_BUTTON,
+    NEXT_SEGMENT_BUTTON,
+    EVENT_LAYERS_BUTTON,
+    AUTO_JUMP_BUTTON,
     BUTTON_MAX,
+    
 
     SPLITTER = 0x20,
     INPUT_SLIDER,
@@ -104,6 +112,11 @@ typedef enum buttonType_e
     SPEED_LABEL,
     TIME_LABEL_ELAPSED,
     TIME_LABEL_REMAINING,
+    META_SLIDER,
+    SEGMENTS_TIME_LABEL,
+    SEGMENTS_TIME_LABEL_ELAPSED,
+    SEGMENTS_TIME_LABEL_REMAINING,
+    EVENTS_EDITOR,
     SPECIAL_MAX,
 
     WIDGET_SPACER = 0x40,
@@ -118,7 +131,8 @@ static const char* const nameL[BUTTON_MAX] = { N_("Play"), N_("Stop"), N_("Open"
     N_("Record"), N_("A->B Loop"), N_("Frame By Frame"), N_("Trickplay Reverse"),
     N_("Step backward" ), N_("Step forward"), N_("Quit"), N_("Random"),
     N_("Loop / Repeat"), N_("Information"), N_("Previous"), N_("Next"),
-    N_("Open subtitles"), N_("Dock fullscreen controller")
+    N_("Open subtitles"), N_("Dock fullscreen controller"), N_("Annotations"),
+    N_("Previous Segment"), N_("Next Segment"), N_("Event Layers"), N_("Auto Jump")
 };
 static const char* const tooltipL[BUTTON_MAX] = { I_PLAY_TOOLTIP,
     N_("Stop playback"), N_("Open a medium"),
@@ -132,7 +146,9 @@ static const char* const tooltipL[BUTTON_MAX] = { I_PLAY_TOOLTIP,
     N_("Random"), N_("Change the loop and repeat modes"), N_("Information"),
     N_("Previous media in the playlist"), N_("Next media in the playlist"),
     N_("Open subtitle file"),
-    N_("Dock/undock fullscreen controller to/from bottom of screen")
+    N_("Dock/undock fullscreen controller to/from bottom of screen"),
+    N_("Open media annotations"), N_("Skip to previous segment"), N_("Skip to next segment"), N_("Open event layers control"),
+    N_("Enable/Disable Auto jump to events")
 };
 static const QString iconL[BUTTON_MAX] ={ ":/toolbar/play_b", ":/toolbar/stop_b",
     ":/toolbar/eject", ":/toolbar/previous_b", ":/toolbar/next_b",
@@ -142,7 +158,8 @@ static const QString iconL[BUTTON_MAX] ={ ":/toolbar/play_b", ":/toolbar/stop_b"
     ":/toolbar/frame", ":/toolbar/reverse", ":/toolbar/skip_back",
     ":/toolbar/skip_fw", ":/toolbar/clear", ":/buttons/playlist/shuffle_on",
     ":/buttons/playlist/repeat_all", ":/menu/info",
-    ":/toolbar/previous_b", ":/toolbar/next_b", ":/toolbar/eject", ":/toolbar/space"
+    ":/toolbar/previous_b", ":/toolbar/next_b", ":/toolbar/eject", ":/toolbar/space",
+    ":/toolbar/annotations", ":/toolbar/dvd_prev", ":/toolbar/dvd_next", ":/prefsmenu/advanced/intf", ":/toolbar/skip_fw"
 };
 
 enum
@@ -180,6 +197,7 @@ protected:
 private:
     static void setupButton( QAbstractButton * );
     QFrame *discFrame();
+    QFrame *eventsFrame();
     QFrame *telexFrame();
     void applyAttributes( QToolButton *, bool b_flat, bool b_big );
 
@@ -237,6 +255,14 @@ signals:
     void advancedControlsToggled( bool );
 };
 
+/* MetaLayers bar */
+class MetaLayerControlsWidget : public AbstractController
+{
+    Q_OBJECT
+public:
+    MetaLayerControlsWidget( intf_thread_t * , QWidget *_parent = 0 );
+};
+
 
 /* to trying transparency with fullscreen controller on windows enable that */
 /* it can be enabled on-non windows systems,
diff --git a/modules/gui/qt4/dialogs/toolbar.cpp b/modules/gui/qt4/dialogs/toolbar.cpp
index 34e0924..6d623f0 100644
--- a/modules/gui/qt4/dialogs/toolbar.cpp
+++ b/modules/gui/qt4/dialogs/toolbar.cpp
@@ -356,6 +356,13 @@ WidgetListing::WidgetListing( intf_thread_t *p_intf, QWidget *_parent )
             }
             widgetItem->setText( qtr("Time Slider") );
             break;
+        case META_SLIDER:
+            {
+                MetaSlider *slider = new MetaSlider( Qt::Horizontal, this );
+                widget = slider;
+            }
+            widgetItem->setText( qtr("MetaLayer Slider") );
+            break;
         case VOLUME:
             {
                 SoundWidget *snd = new SoundWidget( this, p_intf,
@@ -380,6 +387,21 @@ WidgetListing::WidgetListing( intf_thread_t *p_intf, QWidget *_parent )
             }
             widgetItem->setText( qtr("Time") );
             break;
+        case SEGMENTS_TIME_LABEL:
+            {
+                QLabel *segmentsTimeLabel = new QLabel( "12:42/2:12:42", this );
+                widget = segmentsTimeLabel;
+            }
+            widgetItem->setText( qtr("Segments Time") );
+            break;
+        case SEGMENTS_TIME_LABEL_ELAPSED:
+            widget = new QLabel( "2:42", this );
+            widgetItem->setText( qtr("Elapsed Segments Time") );
+            break;
+        case SEGMENTS_TIME_LABEL_REMAINING:
+            widget = new QLabel( "-2:42", this );
+            widgetItem->setText( qtr("Total/Remaining Segments time") );
+            break;
         case MENU_BUTTONS:
             {
                 QWidget *discFrame = new QWidget( this );
@@ -464,6 +486,38 @@ WidgetListing::WidgetListing( intf_thread_t *p_intf, QWidget *_parent )
             widget = new QLabel( "-2:42", this );
             widgetItem->setText( qtr("Total/Remaining time") );
             break;
+        case EVENTS_EDITOR:
+            {
+                //
+                QWidget *eventsFrame = new QWidget( this );
+                //discFrame->setLineWidth( 1 );
+                QHBoxLayout *eventsLayout = new QHBoxLayout( eventsFrame );
+                eventsLayout->setSpacing( 0 ); eventsLayout->setMargin( 0 );
+
+                QToolButton *newEventButton = new QToolButton( eventsFrame );
+                newEventButton->setIcon( QIcon( ":/util/create_event" ) );
+                newEventButton->setToolTip( qtr("Create new event") );
+                eventsLayout->addWidget( newEventButton );
+
+                QToolButton *editEventButton = new QToolButton( eventsFrame );
+                editEventButton->setIcon( QIcon( ":/util/edit_event" ) );
+                editEventButton->setToolTip( qtr("Edit existing event") );
+                eventsLayout->addWidget( editEventButton );
+
+                QToolButton *deleteEventButton = new QToolButton( eventsFrame );
+                deleteEventButton->setIcon( QIcon( ":/util/delete_event" ) );
+                deleteEventButton->setToolTip( qtr("Delete event") );
+                eventsLayout->addWidget( deleteEventButton );
+
+                QToolButton *saveEventsData = new QToolButton( eventsFrame );
+                saveEventsData->setIcon( QIcon( ":/util/save_events" ) );
+                saveEventsData->setToolTip( qtr("Save events data to file") );
+                eventsLayout->addWidget( saveEventsData );
+
+                widget = eventsFrame;
+            }
+            widgetItem->setText( qtr("Events Editor") );
+            break;
         default:
             msg_Warn( p_intf, "This should not happen %i", i );
             break;
-- 
1.8.3.msysgit.0

